/*
** Module   : tty.keymap.js
** Abstract : keys definition and mapping.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
** 002 MAG 20140228 Add javadoc.
*/

/**
 * Global definitions for common key values.
 */
keys = 
{
   BACKSPACE: 8,
   TAB: 9,
   CLEAR: 12,
   ENTER: 13,
   SHIFT: 16,
   CTRL: 17,
   ALT: 18,
   PAUSE: 19,
   CAPS_LOCK: 20,
   ESCAPE: 27,
   SPACE: 32,
   PAGE_UP: 33,
   PAGE_DOWN: 34,
   END: 35,
   HOME: 36,
   LEFT_ARROW: 37,
   UP_ARROW: 38,
   RIGHT_ARROW: 39,
   DOWN_ARROW: 40,
   INSERT: 45,
   DELETE: 46,
   HELP: 47,
   LEFT_WINDOW: 91,
   RIGHT_WINDOW: 92,
   SELECT: 93,
   NUMPAD_0: 96,
   NUMPAD_1: 97,
   NUMPAD_2: 98,
   NUMPAD_3: 99,
   NUMPAD_4: 100,
   NUMPAD_5: 101,
   NUMPAD_6: 102,
   NUMPAD_7: 103,
   NUMPAD_8: 104,
   NUMPAD_9: 105,
   NUMPAD_MULTIPLY: 106,
   NUMPAD_PLUS: 107,
   NUMPAD_ENTER: 108,
   NUMPAD_MINUS: 109,
   NUMPAD_PERIOD: 110,
   NUMPAD_DIVIDE: 111,
   F1: 112,
   F2: 113,
   F3: 114,
   F4: 115,
   F5: 116,
   F6: 117,
   F7: 118,
   F8: 119,
   F9: 120,
   F10: 121,
   F11: 122,
   F12: 123,
   F13: 124,
   F14: 125,
   F15: 126,
   NUM_LOCK: 144,
   SCROLL_LOCK: 145,
   UP_DPAD: 175,
   DOWN_DPAD: 176,
   LEFT_DPAD: 177,
   RIGHT_DPAD: 178
};

tty.keymap = (function()
{
   /* exports object */
   var my = {};
   
   /* normal key mapping */   
   my.xlate = {};
   
   /* ALT key mapping */
   my.xlateAlt = {};         
       
   /**
    * Init function.
    */
   my.init = function()
   {
      // Normal keys
      my.xlate[keys.BACKSPACE] = 8;
      my.xlate[keys.TAB] = 9;
      my.xlate[keys.ENTER] = 13;
      my.xlate[keys.UP_ARROW] = 501;
      my.xlate[keys.DOWN_ARROW] = 502;
      my.xlate[keys.RIGHT_ARROW] = 503;
      my.xlate[keys.LEFT_ARROW] = 504;
      my.xlate[keys.HOME] = 505;
      my.xlate[keys.END] = 506;
      my.xlate[keys.PAGE_UP] = 507;
      my.xlate[keys.PAGE_DOWN] = 508;
      my.xlate[keys.INSERT] = 510;
      my.xlate[keys.DELETE] = 127;     
      my.xlate[keys.F1] = 301;
      my.xlate[keys.F2] = 302;
      my.xlate[keys.F3] = 303;
      my.xlate[keys.F4] = 304;
      my.xlate[keys.F5] = 305;
      my.xlate[keys.F6] = 306;
      my.xlate[keys.F7] = 307;
      my.xlate[keys.F8] = 308;
      my.xlate[keys.F9] = 309;
      my.xlate[keys.F10] = 310;
      my.xlate[keys.F11] = 311;
      my.xlate[keys.F12] = 312;
      
      // ALT-keys
      my.xlateAlt[keys.UP_ARROW] = 559;      
      my.xlateAlt[keys.DOWN_ARROW] = 518;
      my.xlateAlt[keys.RIGHT_ARROW] = 553;
      my.xlateAlt[keys.LEFT_ARROW] = 538;
      my.xlateAlt[keys.PAGE_UP] = 548;
      my.xlateAlt[keys.PAGE_DOWN] = 543;
      my.xlateAlt[keys.INSERT] = 533;
      my.xlateAlt[keys.DELETE] = 127;     
      my.xlateAlt[keys.HOME] = 528;
      my.xlateAlt[keys.END] = 523;
      my.xlateAlt[keys.F1] = 313;
      my.xlateAlt[keys.F2] = 314;
      my.xlateAlt[keys.F3] = 315;
      my.xlateAlt[keys.F4] = 316;
      my.xlateAlt[keys.F5] = 317;
      my.xlateAlt[keys.F6] = 318;
      my.xlateAlt[keys.F7] = 319;
      my.xlateAlt[keys.F8] = 320;
      my.xlateAlt[keys.F9] = 321;
      my.xlateAlt[keys.F10] = 322;
      my.xlateAlt[keys.F11] = 323;
      my.xlateAlt[keys.F12] = 324;
   };
   
   /**
    * Map scan code to ASCII printable,
    * 
    * @param {event} evt event object.
    * 
    * @return {number}   mapped key code.
    */
   my.mapKey = function(evt)
   {
      var key = evt.keyCode;
      
      if (tty.isFirefox)
      {
         switch(key)
         {
            case 59:
               key = 186; //; : 
               break;
            case 107:
               key = 187; //= + 
               break;
            case 109:
               key = 189; //- _
               break;
         }
      }
      
      if (evt.shiftKey)
      {
         switch(key)
         {
            case 48:
               return 41; //)
            case 49:
               return 33; //!
            case 50:
               return 64; //@
            case 51:
               return 35; //#
            case 52:
               return 36; //$
            case 53:
               return 37; //%
            case 54:
               return 94; //^
            case 55:
               return 38; //&
            case 56:
               return 42; //*
            case 57:
               return 40; //(
            case 188:
               return 60; //<
            case 190:
               return 62; //>
            case 191:
               return 63; //?
            case 192:
               return 126; //~
            case 219:
               return 123; //{
            case 220:
               return 124; //|
            case 221:
               return 125; //}
            case 222:
               return 34; //"
            case 186:
               return 58; //:
            case 187:
               return 43; //+
            case 189:
               return 95; //_
         }
      }
      else
      {
         // 0...9
         if (key >= 48 && key <= 57)
         {
            return key;
         }

         switch(key)
         {
            case 188:
               return 44; //,
            case 190:
               return 46; //.
            case 191:
               return 47; ///
            case 192:
               return 96; //`
            case 219:
               return 91; //[
            case 220:
               return 92; //\
            case 221:
               return 93; //]
            case 222:
               return 34; //'
            case 186:
               return 59; //;
            case 187:
               return 61; //=
            case 189:
               return 45; //-
            // numpad
            case 96:
            case 97:
            case 98:
            case 99:
            case 100:
            case 101:
            case 102:
            case 103:
            case 104:
            case 105:
               return key - 48; //0...9
            case 106:
               return 42; //*
            case 107:
               return 43; //+
            case 109:
               return 45; //-
            case 110:
               return 46; //.
            case 111:
               return 47; ///
         }
      }
      
      // A...Z
      if (key >= 65 && key <= 90)
      {
         return key;
      }
      
      return -1;
   };
      
   /**
    * Process CTRL key.
    * 
    * @param {object} evt  event object
    * @param {mumber} code char code
    * 
    * @returns {number}  mapped code
    */
   my.processCtrl = function(evt, code)
   {
      var ctrlCode = -1;
      
      if (evt.ctrlKey)
      {         
         if (code === 50) //'2'
         {
            ctrlCode = 0;
         }
         else if (code === 54)  //'6'
         {
            ctrlCode = 30;
         }
         else if (code === 45) //'-'
         {
            ctrlCode = 31;
         }
         else if (code === 92) //'\'
         {           
            ctrlCode = 28;       
         }
         else if (code === 93) //']'
         {           
            ctrlCode = 29;       
         }
         else
         {
            if (code >= 32 && code < 127)
            {
               ctrlCode = code & 0xffbf;
               // VK_ESCAPE - NOT DELIVERED, 4GL doesn't ever return this.                        
               if (ctrlCode === keys.ESCAPE)
               {
                  ctrlCode = -1;
               }
            }
         }
      }
      
      return ctrlCode;
   };
      
   /* exports */
   return my;         
}());
